package net.sourceforge.nattable.extension.glazedlists;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import net.sourceforge.nattable.command.DisposeResourcesCommand;
import net.sourceforge.nattable.command.ILayerCommand;
import net.sourceforge.nattable.layer.AbstractLayerTransform;
import net.sourceforge.nattable.layer.IUniqueIndexLayer;
import net.sourceforge.nattable.layer.event.ILayerEvent;
import net.sourceforge.nattable.layer.event.PropertyUpdateEvent;
import net.sourceforge.nattable.layer.event.RowStructuralRefreshEvent;

import org.eclipse.swt.widgets.Display;

import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.event.ListEvent;
import ca.odell.glazedlists.event.ListEventListener;

/**
 * This layer acts as the event listener for:
 * <ol>
 *    <li>Glazed list events - {@link ListEvent}
 *    <li>Bean updates - PropertyChangeEvent(s)
 * </ol>
 * GlazedLists Events are conflated at a 100ms interval i.e a single {@link RowStructuralRefreshEvent}
 * is fired for any number of GlazedLists events received during that interval.<br/>
 *
 * PropertyChangeEvent(s) are propagated immediately as a {@link PropertyUpdateEvent}.
 *
 * @param <T> Type of the bean in the backing list.
 */
public class GlazedListsEventLayer<T> extends AbstractLayerTransform implements IUniqueIndexLayer, ListEventListener<T>, PropertyChangeListener{

	private final IUniqueIndexLayer underlyingLayer;
	private EventList<T> eventList;
	private boolean testMode = false;
	private final ScheduledExecutorService threadPool;
	private boolean eventsToProcess = false;

	public GlazedListsEventLayer(IUniqueIndexLayer underlyingLayer, EventList<T> eventList) {
	    super(underlyingLayer);
	    this.underlyingLayer = underlyingLayer;
		this.eventList = eventList;

		this.eventList.addListEventListener(this);

		// Start the event conflation thread
		threadPool = Executors.newScheduledThreadPool(1);
		threadPool.scheduleAtFixedRate(getEventNotifier(), 0L, 100L, TimeUnit.MILLISECONDS);
	}

	/**
	 * Fires a NatTable refresh event, if any glazed list events have occurred.
	 */
	protected Runnable getEventNotifier() {
		return new Runnable(){
			public void run() {
				if(eventsToProcess){
					fireEventFromSWTDisplayThread(new RowStructuralRefreshEvent(getUnderlyingLayer()));
				}
				eventsToProcess = false;
			}
		};
	}

	/**
	 * Glazed list event handling.
	 */
	public void listChanged(ListEvent<T> event) {
		eventsToProcess = true;
	}

	/**
	 * Object property updated event
	 */
	@SuppressWarnings("unchecked")
	public void propertyChange(PropertyChangeEvent event) {
		// We can cast since we know that the EventList is of type T
		PropertyUpdateEvent<T> updateEvent = new PropertyUpdateEvent<T>(this,
													  (T)event.getSource(),
	                                                  event.getPropertyName(),
	                                                  event.getOldValue(),
	                                                  event.getNewValue());
		fireEventFromSWTDisplayThread(updateEvent);
	}

	/**
	 * These update events are likely to cause a repaint on NatTable.
	 * If these are not thrown from the SWT Display thread, SWT
	 * will throw an Exception. Painting can only be triggered from the
	 * SWT Display thread.
	 */
	protected void fireEventFromSWTDisplayThread(final ILayerEvent event) {
		if (!testMode) {
			Display.getDefault().asyncExec(new Runnable() {
				public void run() {
					fireLayerEvent(event);
				}
			});
		} else {
			fireLayerEvent(event);
		}
	}

	@Override
	public boolean doCommand(ILayerCommand command) {
		if(command instanceof DisposeResourcesCommand){
			threadPool.shutdownNow();
		}
		return super.doCommand(command);
	}

	public boolean isDisposed() {
		return threadPool.isShutdown();
	}

	/**
	 * @param newEventList the {@link EventList} to listen on.
	 */
	public void setEventList(EventList<T> newEventList){
		eventList.removeListEventListener(this);
		eventList = newEventList;
		eventList.addListEventListener(this);
	}

	public void setTestMode(boolean testMode) {
		this.testMode = testMode;
	}

	// Columns

	public int getColumnPositionByIndex(int columnIndex) {
		return underlyingLayer.getColumnPositionByIndex(columnIndex);
	}

	// Rows

	public int getRowPositionByIndex(int rowIndex) {
		return underlyingLayer.getRowPositionByIndex(rowIndex);
	}
}