package net.sourceforge.nattable.group;

import java.util.List;
import java.util.Properties;

import net.sourceforge.nattable.NatTable;
import net.sourceforge.nattable.grid.GridRegion;
import net.sourceforge.nattable.group.config.DefaultRowGroupHeaderLayerConfiguration;
import net.sourceforge.nattable.layer.AbstractLayerTransform;
import net.sourceforge.nattable.layer.DataLayer;
import net.sourceforge.nattable.layer.ILayer;
import net.sourceforge.nattable.layer.LabelStack;
import net.sourceforge.nattable.layer.SizeConfig;
import net.sourceforge.nattable.layer.cell.LayerCell;
import net.sourceforge.nattable.layer.event.ColumnStructuralRefreshEvent;
import net.sourceforge.nattable.painter.layer.CellLayerPainter;
import net.sourceforge.nattable.painter.layer.ILayerPainter;
import net.sourceforge.nattable.selection.SelectionLayer;
import net.sourceforge.nattable.style.DisplayMode;

/**
 * Adds the Row grouping functionality to the row headers.<br/>
 * Also persists the state of the row groups when {@link NatTable#saveState()} is invoked.<br/>
 * <br/>
 * Internally uses the {@link RowGroupModel} to track the row groups.<br/>
 * @see RowGroupGridExample
 */
public class RowGroupHeaderLayer extends AbstractLayerTransform {

	private final SizeConfig columnWidthConfig = new SizeConfig(DataLayer.DEFAULT_COLUMN_WIDTH);
	private final RowGroupModel model;
	private final ILayer rowHeaderLayer;
	private final ILayerPainter layerPainter = new CellLayerPainter();
	
	public RowGroupHeaderLayer(ILayer rowHeaderLayer, SelectionLayer selectionLayer, RowGroupModel rowGroupModel) {
		this(rowHeaderLayer, selectionLayer, rowGroupModel, true);
	}
		
	public RowGroupHeaderLayer(ILayer rowHeaderLayer, SelectionLayer selectionLayer, RowGroupModel rowGroupModel, boolean useDefaultConfiguration) {
		super(rowHeaderLayer);
		this.rowHeaderLayer = rowHeaderLayer;
		this.model = rowGroupModel;

		if( useDefaultConfiguration ) {
			addConfiguration( new DefaultRowGroupHeaderLayerConfiguration(rowGroupModel) );
		}
	}
	
	public RowGroupModel getModel() {
		return this.model;
	}
	
	// Persistence
	
	@Override
	public void saveState(String prefix, Properties properties) {
		super.loadState(prefix, properties);
		model.loadState(prefix, properties);
		fireLayerEvent(new ColumnStructuralRefreshEvent(this));
	}
	
	@Override
	public void loadState(String prefix, Properties properties) {
		super.saveState(prefix, properties);
		model.saveState(prefix, properties);
	}
	
	// Configuration
	
	@Override
	public ILayerPainter getLayerPainter() {
		return layerPainter;
	}
	
	// Horizontal features

	// Columns
	
	@Override
	public int getColumnCount() {
		return rowHeaderLayer.getColumnCount() + 1;
	}
	
	@Override
	public int getPreferredColumnCount() {
		return rowHeaderLayer.getPreferredColumnCount() + 1;
	}
	
	@Override
	public int getColumnIndexByPosition(int columnPosition) {
		if( columnPosition == 0 ) {
			return columnPosition;
		} else {
			return rowHeaderLayer.getColumnIndexByPosition(columnPosition - 1);
		}
	}
	
	// Width
	
	@Override
	public int getWidth() {
		return columnWidthConfig.getAggregateSize(1) + rowHeaderLayer.getWidth();
	}
	
	@Override
	public int getPreferredWidth() {		
		return columnWidthConfig.getAggregateSize(1) + rowHeaderLayer.getPreferredWidth();
	}
	
	@Override
	public int getColumnWidthByPosition(int columnPosition) {
		if( columnPosition == 0 ) {
			return columnWidthConfig.getSize(columnPosition);
		} else {
			return rowHeaderLayer.getColumnWidthByPosition(columnPosition - 1);
		}
	}
	
	public void setColumnWidth(int columnWidth) {
		this.columnWidthConfig.setSize(0, columnWidth);
	}
	
	// Column resize
	
	@Override
	public boolean isColumnPositionResizable(int columnPosition) {
		if( columnPosition == 0 ) {
			return columnWidthConfig.isPositionResizable(columnPosition);
		} else {
			return rowHeaderLayer.isRowPositionResizable(columnPosition - 1);
		}
	}
	
	// X
	
	@Override
	public int getColumnPositionByX(int x) {
		int col0Width = getColumnWidthByPosition(0);
		if( x < col0Width ) {
			return 0;
		} else {
			return 1 + rowHeaderLayer.getColumnPositionByX(x - col0Width);
		}
	}
	
	@Override
	public int getStartXOfColumnPosition(int columnPosition) {
		if( columnPosition == 0 ) {
			return columnWidthConfig.getAggregateSize(columnPosition);
		} else {
			return getColumnWidthByPosition(0) + rowHeaderLayer.getStartXOfColumnPosition(columnPosition - 1);
		}
	}
	
	// Cell features
	
	/**
	 * If a cell belongs to a column group:
	 * 	 column position - set to the start position of the group
	 * 	 span - set to the width/size of the row group
	 *
	 * NOTE: gc.setClip() is used in the CompositeLayerPainter to ensure that partially visible
	 * Column group header cells are rendered properly.
	 */
	@Override
	public LayerCell getCellByPosition(int columnPosition, int rowPosition) {		
		int bodyRowIndex = getRowIndexByPosition(rowPosition);

		// Row group header cell
		if (model.isPartOfAGroup(bodyRowIndex)) {
			if (columnPosition == 0) {
				return new LayerCell(
						this,
						columnPosition, getStartPositionOfGroup(rowPosition),
						columnPosition, rowPosition,
						1, getRowSpan(rowPosition)
				);
			}
		}
		
		return new LayerCell(this, columnPosition, rowPosition);
	}
	
	/**
	 * Calculates the span of a cell in a Row Group.
	 * Takes into account collapsing and hidden rows in the group.
	 *
	 * @param selectionLayerColumnPosition of any row belonging to the group
	 */
	protected int getRowSpan(int columnPosition) {
		int columnIndex = getRowIndexByPosition(columnPosition);
		if (model.isCollapsed(columnIndex)) {
			return 1;
		} else {
			int startPositionOfGroup = getStartPositionOfGroup(columnPosition);
			int sizeOfGroup = model.sizeOfGroup(columnIndex);
			int endPositionOfGroup = startPositionOfGroup + sizeOfGroup;
			List<Integer> rowIndexesInGroup = model.getRowIndexesInGroup(columnIndex);

			for (int i = startPositionOfGroup; i < endPositionOfGroup; i++) {
				int index = getRowIndexByPosition(i);
				if (!rowIndexesInGroup.contains(Integer.valueOf(index))) {
					sizeOfGroup--;
				}
			}
			return sizeOfGroup;
		}
	}
	
	/**
	 * Figures out the start position of the group.
	 *
	 * @param selectionLayerColumnPosition of any column belonging to the group
	 * @return first position of the column group
	 */
	private int getStartPositionOfGroup(int rowPosition) {
		int bodyRowIndex = getRowIndexByPosition(rowPosition);
		int leastPossibleStartPositionOfGroup = Math.max(0, (rowPosition - model.sizeOfGroup(bodyRowIndex)));
		int i = 0;
		for (i = leastPossibleStartPositionOfGroup; i < rowPosition; i++) {
			if (RowGroupUtils.isInTheSameGroup(getRowIndexByPosition(i), bodyRowIndex, model)) {
				break;
			}
		}
		return i;
	}
	
	@Override
	public String getDisplayModeByPosition(int columnPosition, int rowPosition) {
		int rowIndex = getRowIndexByPosition(rowPosition);
		if( columnPosition == 0 && model.isPartOfAGroup(rowIndex) ) {
			return DisplayMode.NORMAL;
		} else {
			return rowHeaderLayer.getDisplayModeByPosition(columnPosition - 1, rowPosition);
		}
	}
	
	@Override
	public LabelStack getConfigLabelsByPosition(int columnPosition, int rowPosition) {
		int rowIndex = getRowIndexByPosition(rowPosition);
		if( columnPosition == 0 && model.isPartOfAGroup(rowIndex) ) {
			return new LabelStack(GridRegion.ROW_GROUP_HEADER);
		} else {
			return rowHeaderLayer.getConfigLabelsByPosition(columnPosition - 1, rowPosition);
		}
	}
	
	@Override
	public Object getDataValueByPosition(int columnPosition, int rowPosition) {
		int rowIndex = getRowIndexByPosition(rowPosition);
		if( columnPosition == 0 ) {
			if( model.isPartOfAGroup(rowIndex) ) {		
				return model.getRowGroupNameForIndex(rowIndex);
			} else {
				return null;
			}
		} else {
			return rowHeaderLayer.getDataValueByPosition(columnPosition - 1, rowPosition);
		}
	}
	
	@Override
	public LabelStack getRegionLabelsByXY(int x, int y) {
		int rowIndex = getRowIndexByPosition(getRowPositionByY(y));
		if (model.isPartOfAGroup(rowIndex) && x < getColumnWidthByPosition(0)) {
			return new LabelStack(GridRegion.ROW_GROUP_HEADER);
		} else {
			return rowHeaderLayer.getRegionLabelsByXY(x - getColumnWidthByPosition(0), y);
		}
	}

	// RowGroupModel delegates

	public void addRowsIndexesToGroup(String colGroupName, int... colIndexes) {
		model.addRowsIndexesToGroup(colGroupName, colIndexes);
	}

	public void collapseRowGroupByIndex(int columnIndex) {
		model.collapse(columnIndex);
	}

	public void clearAllGroups(){
		model.clear();
	}

	/**
	 * @see RowGroupModel#setGroupUnBreakable(int)
	 */
	public void setGroupUnbreakable(int columnIndex){
		model.setGroupUnBreakable(columnIndex);
	}

	public void setGroupAsCollapsed(int i) {
		model.collapse(i);
	}
}
